@@ -14,11 +14,16 @@
 #ifndef SHAREDIALOG_H
 #define SHAREDIALOG_H
 
-#include "accountfwd.h"
+#include "accountstate.h"
+#include "sharepermissions.h"
+
+#include <QPointer>
 #include <QString>
 #include <QDialog>
 #include <QWidget>
 
+class QProgressIndicator;
+
 namespace OCC {
 
 namespace Ui {
@@ -33,29 +38,34 @@ class ShareDialog : public QDialog
     Q_OBJECT
 
 public:
-    explicit ShareDialog(AccountPtr account,
+    explicit ShareDialog(QPointer<AccountState> accountState,
                          const QString &sharePath,
                          const QString &localPath,
-                         bool resharingAllowed,
+                         SharePermissions maxSharingPermissions,
                          QWidget *parent = 0);
     ~ShareDialog();
 
-    void getShares();
-
 private slots:
     void done( int r );
+    void slotMaxSharingPermissionsReceived(const QVariantMap &result);
+    void slotMaxSharingPermissionsError();
     void slotThumbnailFetched(const int &statusCode, const QByteArray &reply);
+    void slotAccountStateChanged(int state);
 
 private:
+
+    void showSharingUi();
+
     Ui::ShareDialog *_ui;
-    AccountPtr _account;
+    QPointer<AccountState> _accountState;
     QString _sharePath;
     QString _localPath;
 
-    bool _resharingAllowed;
+    SharePermissions _maxSharingPermissions;
 
     ShareLinkWidget *_linkWidget;
     ShareUserGroupWidget *_userGroupWidget;
+    QProgressIndicator *_progressIndicator;
 };
 
 }
